//
//  PUPlayOptionsViewController.m
//  BJPlayerManagerUI_Example
//
//  Created by daixijia on 2018/8/2.
//  Copyright © 2018年 oushizishu. All rights reserved.
//

#import "PUPlayOptionsViewController.h"
#import <Masonry.h>
#import <BJPlayerManagerCore/PMAppConfig.h>

@implementation PUPlayOptions

+ (instancetype)shareInstance {
    static PUPlayOptions *shareInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        shareInstance = [[PUPlayOptions alloc] init];
    });
    return shareInstance;
}

- (instancetype)init {
    if (self = [super init]) {
        self.advertisementEnabled = NO;
        self.notAutoStartImageAD = NO;
        self.autoplay = YES;
        self.sliderDragEnabled = YES;
        self.playTimeRecordEnabled = YES;
        self.encryptEnabled = NO;
        self.backgroundAudioEnabled = YES;
        self.playRate = 1.0;
        self.preferredDefinitionList = @[@(-2),@(0),@(1),@(2),@(3),@(4)];
    }
    return self;
}

#pragma mark - <YYModel>

- (void)encodeWithCoder:(NSCoder *)aCoder { [self bjlyy_modelEncodeWithCoder:aCoder]; }
- (nullable instancetype)initWithCoder:(NSCoder *)aDecoder { self = [super init]; return [self bjlyy_modelInitWithCoder:aDecoder]; }
- (id)copyWithZone:(nullable NSZone *)zone { return [self bjlyy_modelCopy]; }
- (NSUInteger)hash { return [self bjlyy_modelHash]; }
- (BOOL)isEqual:(id)object { return [self bjlyy_modelIsEqual:object]; }

@end

static NSInteger initialTag = 1000;

@interface PUPlayOptionsViewController () <UITextFieldDelegate>

@property (nonatomic) UITextField *initialPlaybackTimeTextField, *nameTextField, *numberTextField, *selectDefinitionTextField, *exclusiveDomainTextField, *accessKeyTextField;
@property (nonatomic) UILabel *adLabel, *autoPlayImageADLabel, *sliderLabel, *autoPlayLabel, *encryptLabel, *memoryLabel, *backgroundPlayLabel, *initialPlaybackTimeLabel, *nameLabel, *numberLabel, *definitionLabel, *exclusiveDomainLabel, *accessKeyLabel;
@property (nonatomic) UISwitch *adSwitch, *autoPlayImageADSwitch, *sliderSwitch, *autoPlaySwitch, *encryptSwitch, *memorySwitch, *backgroundPlaySwitch;
@property (nonatomic) NSArray<NSString *> *definitionTitles;
@property (nonatomic) NSArray<UIButton *> *definitionButtons;
@property (nonatomic) NSMutableArray<NSNumber *> *preferredDefinitions;
@property (nonatomic) UIButton *confirmButton;

@end

@implementation PUPlayOptionsViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    [self setupSubviews];
}

- (void)viewWillAppear:(BOOL)animated {
    
    PUPlayOptions *options = [PUPlayOptions shareInstance];
    self.adSwitch.on = options.advertisementEnabled;
    self.autoPlayImageADSwitch.on = !options.notAutoStartImageAD;
    self.autoPlaySwitch.on = options.autoplay;
    self.sliderSwitch.on = options.sliderDragEnabled;
    self.memorySwitch.on = options.playTimeRecordEnabled;
    self.encryptSwitch.on = options.encryptEnabled;
    self.backgroundPlaySwitch.on = options.backgroundAudioEnabled;
    self.nameTextField.text = options.userName;
    self.numberTextField.text = [NSString stringWithFormat:@"%ti", options.userNumber];
    self.accessKeyTextField.text = options.accessKey;
    self.exclusiveDomainTextField.text = options.exclusiveDomain;
    
    // 偏好清晰度
    self.preferredDefinitions = [options.preferredDefinitionList mutableCopy];
    [self updateDefinitionControls];
}

#pragma mark - subViews

- (void)setupSubviews {
    // 确认按钮
    self.confirmButton = ({
        UIButton *button = [[UIButton alloc] init];
        button.backgroundColor = [UIColor redColor];
        button.layer.cornerRadius = 5.0;
        [button setTitle:@"确认" forState:UIControlStateNormal];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [button.titleLabel setFont:[UIFont systemFontOfSize:15.0]];
        [button addTarget:self action:@selector(confirmAction) forControlEvents:UIControlEventTouchUpInside];
        button;
    });
    [self.view addSubview:self.confirmButton];
    [self.confirmButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.view.mas_bottom).offset(-30.0);
        make.left.equalTo(self.view).offset(30.0);
        make.right.equalTo(self.view).offset(-30.0);
        make.height.equalTo(@30.0);
    }];
    
    // scrollView
    UIScrollView *scrollView = [[UIScrollView alloc] init];
    [self.view addSubview:scrollView];
    [scrollView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.right.equalTo(self.view);
        make.bottom.equalTo(self.confirmButton.mas_top).offset(-30.0);
    }];
    
    UIView *contentView = [[UIView alloc] init];
    [scrollView addSubview:contentView];
    [contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(@[self.view, scrollView]);
        make.top.bottom.equalTo(scrollView);
    }];
    
    CGSize labelSize = CGSizeMake(180.0, 30.0);
    
    [contentView addSubview:self.adLabel];
    [self.adLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(contentView).offset(40.0);
        make.left.equalTo(contentView).offset(30.0);
        make.size.mas_equalTo(labelSize);
    }];
    
    [self.view addSubview:self.adSwitch];
    [self.adSwitch mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.adLabel.mas_right).offset(20.0);
        make.top.bottom.equalTo(self.adLabel);
    }];
    
    [self setupLabel:self.autoPlayImageADLabel withView:self.autoPlayImageADSwitch preview:self.adLabel superview:contentView];

    [self setupLabel:self.sliderLabel withView:self.sliderSwitch preview:self.autoPlayImageADLabel superview:contentView];

    [self setupLabel:self.autoPlayLabel withView:self.autoPlaySwitch preview:self.sliderLabel superview:contentView];

    [self setupLabel:self.encryptLabel withView:self.encryptSwitch preview:self.autoPlayLabel superview:contentView];

    [self setupLabel:self.memoryLabel withView:self.memorySwitch preview:self.encryptLabel superview:contentView];

    [self setupLabel:self.backgroundPlayLabel withView:self.backgroundPlaySwitch preview:self.memoryLabel superview:contentView];

    [self setupLabel:self.initialPlaybackTimeLabel withView:self.initialPlaybackTimeTextField preview:self.backgroundPlayLabel superview:contentView];
    
    [self setupLabel:self.accessKeyLabel withView:self.accessKeyTextField preview:self.initialPlaybackTimeLabel superview:contentView];

    [self setupLabel:self.nameLabel withView:self.nameTextField preview:self.accessKeyLabel superview:contentView];

    [self setupLabel:self.numberLabel withView:self.numberTextField preview:self.nameLabel superview:contentView];
    
    [self setupLabel:self.exclusiveDomainLabel withView:self.exclusiveDomainTextField preview:self.numberLabel superview:contentView];
    
    [contentView addSubview:self.definitionLabel];
    [self.definitionLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.adLabel);
        make.top.equalTo(self.exclusiveDomainLabel.mas_bottom).offset(30.0);
        make.size.mas_equalTo(CGSizeMake(100.0, 30.0));
    }];
    
    [contentView addSubview:self.selectDefinitionTextField];
    [self.selectDefinitionTextField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.definitionLabel.mas_right).offset(5.0);
        make.top.bottom.equalTo(self.definitionLabel);
        make.width.greaterThanOrEqualTo(@(80.0));
    }];

    CGFloat leftOffset = 0.0;
    CGSize buttonSize = CGSizeMake(50.0, 25.0);
    for (UIButton *button in self.definitionButtons) {
        [contentView addSubview:button];
        [button mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.definitionLabel.mas_bottom).offset(5.0);
            make.left.equalTo(self.definitionLabel).offset(leftOffset);
            make.bottom.equalTo(contentView);
            make.size.mas_equalTo(buttonSize);
        }];
        leftOffset += buttonSize.width + 5;
    }
}

- (void)setupLabel:(UILabel *)label withView:(UIView *)view preview:(UIView *)preview superview:(UIView *)superview {
    CGSize labelSize = CGSizeMake(180.0, 30.0);
    [superview addSubview:label];
    [superview addSubview:view];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(superview).offset(30.0);
        make.top.equalTo(preview.mas_bottom).offset(30.0);
        make.size.mas_equalTo(labelSize);
    }];
    
    [view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(label.mas_right).offset(20.0);
        make.top.bottom.equalTo(label);
    }];
}

#pragma mark - action

- (void)confirmAction {
    PUPlayOptions *options = [PUPlayOptions shareInstance];
    options.advertisementEnabled = self.adSwitch.on;
    options.notAutoStartImageAD = !self.autoPlayImageADSwitch.on;
    options.autoplay = self.autoPlaySwitch.on;
    options.sliderDragEnabled = self.sliderSwitch.on;
    options.playTimeRecordEnabled = self.memorySwitch.on;
    options.encryptEnabled = self.encryptSwitch.on;
    options.backgroundAudioEnabled = self.backgroundPlaySwitch.on;
    options.initialPlaybackTime = [self.initialPlaybackTimeTextField.text integerValue];
    options.userName = self.nameTextField.text;
    options.userNumber = [self.numberTextField.text integerValue];
    options.exclusiveDomain = self.exclusiveDomainTextField.text;
    options.accessKey = self.accessKeyTextField.text;
    // 偏好清晰度,仅用于下载
    options.preferredDefinitionList = [NSArray arrayWithArray:self.preferredDefinitions];
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)selectionButtonOnClick:(UIButton *)button {
    NSInteger index = button.tag - initialTag;
//    index = index != 0 ?: -2;
    if (button.selected) {
        // 取消
        if ([self.preferredDefinitions containsObject:@(index)]) {
            [self.preferredDefinitions removeObject:@(index)];
        }
        else {
            return;
        }
    }
    else {
        // 选中
        if ([self.preferredDefinitions containsObject:@(index)]) {
            return;
        }
        else {
            [self.preferredDefinitions addObject:@(index)];
        }
    }
    [self updateDefinitionControls];
}

#pragma mark - getters

- (UILabel *)adLabel {
    if (!_adLabel) {
        _adLabel = [self labelWithText:@"片头片尾:(ijk无效)"];
    }
    return _adLabel;
}

- (UISwitch *)adSwitch {
    if (!_adSwitch) {
        _adSwitch = [[UISwitch alloc] init];
    }
    return _adSwitch;
}


- (UILabel *)autoPlayImageADLabel {
    if (!_autoPlayImageADLabel) {
        _autoPlayImageADLabel = [self labelWithText:@"自动播放图片广告"];
    }
    return _autoPlayImageADLabel;
}

- (UISwitch *)autoPlayImageADSwitch {
    if (!_autoPlayImageADSwitch) {
        _autoPlayImageADSwitch = [[UISwitch alloc] init];
    }
    return _autoPlayImageADSwitch;
}

- (UILabel *)sliderLabel {
    if (!_sliderLabel) {
        _sliderLabel = [self labelWithText:@"拖拽进度条:"];
    }
    return _sliderLabel;
}

- (UISwitch *)sliderSwitch {
    if (!_sliderSwitch) {
        _sliderSwitch = [[UISwitch alloc] init];
        _sliderSwitch.on = YES;
    }
    return _sliderSwitch;
}

- (UILabel *)autoPlayLabel {
    if (!_autoPlayLabel) {
        _autoPlayLabel = [self labelWithText:@"自动播放:"];
    }
    return _autoPlayLabel;
}

- (UISwitch *)autoPlaySwitch {
    if (!_autoPlaySwitch) {
        _autoPlaySwitch = [[UISwitch alloc] init];
    }
    return _autoPlaySwitch;
}

- (UILabel *)encryptLabel {
    if (!_encryptLabel) {
        _encryptLabel = [self labelWithText:@"视频加密:"];
    }
    return _encryptLabel;
}

- (UISwitch *)encryptSwitch {
    if (!_encryptSwitch) {
        _encryptSwitch = [[UISwitch alloc] init];
    }
    return _encryptSwitch;
}

- (UILabel *)memoryLabel {
    if (!_memoryLabel) {
        _memoryLabel = [self labelWithText:@"记忆播放:"];
    }
    return _memoryLabel;
}

- (UISwitch *)memorySwitch {
    if (!_memorySwitch) {
        _memorySwitch = [[UISwitch alloc] init];
    }
    return _memorySwitch;
}

- (UILabel *)backgroundPlayLabel {
    if (!_backgroundPlayLabel) {
        _backgroundPlayLabel = [self labelWithText:@"后台播放: "];
    }
    return _backgroundPlayLabel;
}

-  (UISwitch *)backgroundPlaySwitch {
    if (!_backgroundPlaySwitch) {
        _backgroundPlaySwitch = [[UISwitch alloc] init];
    }
    return _backgroundPlaySwitch;
}

- (UILabel *)initialPlaybackTimeLabel {
    if (!_initialPlaybackTimeLabel) {
        _initialPlaybackTimeLabel = [self labelWithText:@"初始播放时间"];
    }
    return _initialPlaybackTimeLabel;
}

- (UITextField *)initialPlaybackTimeTextField {
    if (!_initialPlaybackTimeTextField) {
        _initialPlaybackTimeTextField = ({
            UITextField *textField = [self textFieldWithPlaceholder:@"初始时间"];
            textField.returnKeyType = UIReturnKeyDone;
            textField.delegate = self;
            textField;
        });
    }
    return _initialPlaybackTimeTextField;
}

- (UILabel *)accessKeyLabel {
    if (!_accessKeyLabel) {
        _accessKeyLabel = [self labelWithText:@"鉴权密钥"];
    }
    return _accessKeyLabel;
}

- (UITextField *)accessKeyTextField {
    if (!_accessKeyTextField) {
        _accessKeyTextField = ({
            UITextField *textField = [self textFieldWithPlaceholder:@"第三方鉴权密钥"];
            textField.returnKeyType = UIReturnKeyDone;
            textField.delegate = self;
            textField;
        });
    }
    return _accessKeyTextField;
}

- (UILabel *)nameLabel {
    if (!_nameLabel) {
        _nameLabel = [self labelWithText:@"用户名"];
    }
    return _nameLabel;
}

- (UITextField *)nameTextField {
    if (!_nameTextField) {
        _nameTextField = ({
            UITextField *textField = [self textFieldWithPlaceholder:@"上报统计"];
            textField.returnKeyType = UIReturnKeyDone;
            textField.delegate = self;
            textField;
        });
    }
    return _nameTextField;
}

- (UILabel *)numberLabel {
    if (!_numberLabel) {
        _numberLabel = [self labelWithText:@"用户编号"];
    }
    return _numberLabel;
}

- (UITextField *)numberTextField {
    if (!_numberTextField) {
        _numberTextField = ({
            UITextField *textField = [self textFieldWithPlaceholder:@"上报统计"];
            textField.keyboardType = UIKeyboardTypeNumberPad;
            textField.returnKeyType = UIReturnKeyDone;
            textField.delegate = self;
            textField;
        });
    }
    return _numberTextField;
}

- (UILabel *)exclusiveDomainLabel {
    if (!_exclusiveDomainLabel) {
        _exclusiveDomainLabel = [self labelWithText:@"专属域名"];
    }
    return _exclusiveDomainLabel;
}

- (UITextField *)exclusiveDomainTextField {
    if (!_exclusiveDomainTextField) {
        _exclusiveDomainTextField = ({
            NSString *string = [[[PMAppConfig sharedInstance].baseURLString.pathComponents lastObject] componentsSeparatedByString:@"."].firstObject;
            UITextField *textField = [self textFieldWithPlaceholder:string];
            textField.returnKeyType = UIReturnKeyDone;
            textField.delegate = self;
            textField;
        });
    }
    return _exclusiveDomainTextField;
}

-  (UILabel *)definitionLabel {
    if (!_definitionLabel) {
        _definitionLabel = [self labelWithText:@"偏好清晰度"];
    }
    return _definitionLabel;
}

- (UITextField *)selectDefinitionTextField {
    if (!_selectDefinitionTextField) {
        _selectDefinitionTextField = ({
            UITextField *textField = [self textFieldWithPlaceholder:@"优先级递减"];
            textField.font = [UIFont systemFontOfSize:14.0];
            textField.enabled = NO;
            textField;
        });
    }
    return _selectDefinitionTextField;
}

- (NSArray<UIButton *> *)definitionButtons {
    if (!_definitionButtons) {
        _definitionButtons = ({
            NSMutableArray *array = [NSMutableArray array];
            for (int i = 0; i < self.definitionTitles.count; i ++) {
                NSString *title = self.definitionTitles[i];
                UIButton *definitionButton = [self selectionButtonWithTitle:title];
                if (i == 0) {
                    definitionButton.tag = initialTag - 2;
                }
                else {
                    definitionButton.tag = initialTag + i - 1;
                }
                [array addObject:definitionButton];
            }
            array;
        });
    }
    return _definitionButtons;
}

- (NSMutableArray<NSNumber *> *)preferredDefinitions {
    if (!_preferredDefinitions) {
        _preferredDefinitions = [NSMutableArray array];
    }
    return _preferredDefinitions;
}

- (NSArray<NSString *> *)definitionTitles {
    if (!_definitionTitles) {
        _definitionTitles = @[@"音频", @"标清", @"高清", @"超清", @"720P", @"1080P"];
    }
    return _definitionTitles;
}

#pragma mark - <UITextFieldDelegate>

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

#pragma mark - private

- (UITextField *)textFieldWithPlaceholder:(NSString *)placeholder {
    UITextField *textField = [[UITextField alloc] init];
    textField.placeholder = placeholder;
    textField.layer.borderColor = [UIColor grayColor].CGColor;
    textField.layer.borderWidth = 0.5;
    textField.layer.cornerRadius = 5.f;
    return textField;
}

- (UILabel *)labelWithText:(NSString *)text {
    UILabel *label = [[UILabel alloc] init];
    label.text = [NSString stringWithFormat:@" %@", text];
    label.layer.borderColor = [UIColor blackColor].CGColor;
    label.layer.cornerRadius = 3.0;
    label.layer.borderWidth = 0.5;
    return label;
}

- (UIButton *)selectionButtonWithTitle:(NSString *)title {
    UIButton *button = [[UIButton alloc] init];
    button.titleLabel.font = [UIFont systemFontOfSize:15.0];
    [button setTitle:title forState:UIControlStateNormal];
    [button setTitleColor:[UIColor lightGrayColor] forState:UIControlStateNormal];
    [button setTitleColor:[UIColor blueColor] forState:UIControlStateSelected];
    button.layer.borderWidth = 1.0;
    button.layer.borderColor = [UIColor lightGrayColor].CGColor;
    button.layer.cornerRadius = 3.0;
    [button addTarget:self action:@selector(selectionButtonOnClick:) forControlEvents:UIControlEventTouchUpInside];
    return button;
}

- (void)updateDefinitionControls {
    for (UIButton *button in self.definitionButtons) {
        button.selected = NO;
        button.layer.borderColor = [UIColor grayColor].CGColor;
    }
    
    NSString *definitionString = self.preferredDefinitions.count? @" " : nil;
    for (NSNumber *number in self.preferredDefinitions) {
        NSInteger index = [number integerValue];
        if (index == -2) {
            index = 0;
        }
        else {
            index++;
        }
        if (index >= 0 && index < self.definitionTitles.count) {
            NSString *title = self.definitionTitles[index];
            definitionString = [NSString stringWithFormat:@"%@ %@", definitionString, title];
        }
        
        if (index >= 0 && index < self.definitionButtons.count) {
            UIButton *button = self.definitionButtons[index];
            button.selected = YES;
            button.layer.borderColor = [UIColor blueColor].CGColor;
        }
        
    }
    self.selectDefinitionTextField.text = definitionString;
}

@end
