//
//  PUPlayViewController.m
//  BJHL-VideoPlayer-UI
//
//  Created by DLM on 2017/4/26.
//  Copyright © 2017年 杨健. All rights reserved.
//

#import <BJPlayerManagerCore/BJPlayerManagerCore.h>
#import <BJPlayerManagerUI/BJPlayerManagerUI.h>
#import <BJPlayerManagerUI/BJPUMacro.h>
#import <ReactiveObjC/ReactiveObjC.h>
#import <Masonry/Masonry.h>
#import <BJPlayerManagerCore/PMReport.h>
#import <BJLiveBase/NSObject+BJLObserving.h>

#import "PUPlayViewController.h"
#import "PUPlayOptionsViewController.h"

@interface PUPlayViewController ()

@property (nonatomic) BOOL isLocal;

// online
@property (nonatomic) NSString *vid;
@property (nonatomic) NSString *token;

// local
@property (nonatomic) NSString *path;
@property (nonatomic) NSInteger definitionType;

// player
@property (nonatomic, readwrite) UIViewController<BJPUViewControllerProtocol> *playerUIVC;
@property (nonatomic, readwrite) PUPlayerType playerType;

@end

@implementation PUPlayViewController

- (void)dealloc {
    [self bjl_stopAllKeyValueObserving];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (instancetype)initWithVid:(NSString *)vid
                      token:(NSString *)token
                 playerType:(PUPlayerType)playerType {
    self = [super init];
    if (self) {
        _isLocal = NO;
        _vid = vid;
        _token = token;
        _playerType = playerType;
    }
    return self;
}

- (instancetype)initWithPath:(NSString *)path definitionType:(NSInteger)definitionType playerType:(PUPlayerType)playerType {
    if (self = [super init]) {
        _isLocal = YES;
        _path = path;
        _definitionType = definitionType;
        _playerType = playerType;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupSubviews];
    [self addNotify];
}

- (void)setupSubviews {
    self.view.backgroundColor = [UIColor whiteColor];
    [self setupPlayerVC];
}

- (void)setupPlayerVC {
    PUPlayOptions *options = [PUPlayOptions shareInstance];
    [[PMAppConfig sharedInstance] setPrivateDomainPrefix:options.exclusiveDomain];
    [PMAppConfig sharedInstance].notAutoStartImageAD = options.notAutoStartImageAD;
    [PMAppConfig sharedInstance].isNeedAD = options.advertisementEnabled;
    
    if (self.isLocal) {
        if (self.playerType == PUPlayerType_AVPlayer) {
            self.playerUIVC = [[BJPUViewController alloc] init];
        }
        else {
            self.playerUIVC = [[BJPIJKViewController alloc] initWithNeedBackgroundModes:options.backgroundAudioEnabled];
        }
        
        self.playerUIVC.playerManager.playTimeRecordEnabled = options.playTimeRecordEnabled;
        self.playerUIVC.playerManager.ignorableRemainingTimeInterval = 4.0;
        // 如果开启了记忆播放，设置无效
        self.playerUIVC.playerManager.initialPlaybackTime = options.initialPlaybackTime;
        // 设置第三方密钥
        self.playerUIVC.playerManager.accessKey = options.accessKey;
        
        [self.playerUIVC setSmallScreenFrame:CGRectMake(0, 64, BJPUScreenWidth, BJPUScreenWidth*9/16)];
        [self.view addSubview:self.playerUIVC.view];
        UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
        if (orientation == UIInterfaceOrientationLandscapeLeft || orientation == UIInterfaceOrientationLandscapeRight) {
            [self.playerUIVC setScreenType:BJPUScreenType_Full];
            self.navigationController.navigationBarHidden = YES;
        }
        
        [self.playerUIVC playWithVideoPath:self.path definitionType:self.definitionType];
        [self addChildViewController:self.playerUIVC];
    }
    else {
        if (self.playerType == PUPlayerType_AVPlayer) {
            self.playerUIVC = [[BJPUViewController alloc] init];
            
        }
        else {
            //!!!:如果希望ijk播放器支持后台播放,除了打开Background Modes, 还需要调用该初始化方法设置为YES, 默认是不支持后台播放的
            self.playerUIVC = [[BJPIJKViewController alloc] initWithNeedBackgroundModes:options.backgroundAudioEnabled];
        }
        [self.playerUIVC sliderMayDrag:options.sliderDragEnabled];
        self.playerUIVC.playerManager.playTimeRecordEnabled = options.playTimeRecordEnabled;
        self.playerUIVC.playerManager.ignorableRemainingTimeInterval = 4.0;
        // 如果开启了记忆播放，设置无效
        self.playerUIVC.playerManager.initialPlaybackTime = options.initialPlaybackTime;
        // 设置第三方密钥
        self.playerUIVC.playerManager.accessKey = options.accessKey;
        
        [self.playerUIVC setSmallScreenFrame:CGRectMake(0, 64, BJPUScreenWidth, BJPUScreenWidth*9/16)];
        [self.view addSubview:self.playerUIVC.view];
        UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
        if (orientation == UIInterfaceOrientationLandscapeLeft || orientation == UIInterfaceOrientationLandscapeRight) {
            [self.playerUIVC setScreenType:BJPUScreenType_Full];
            self.navigationController.navigationBarHidden = YES;
        }
        [self addChildViewController:self.playerUIVC];
        if (self.playerType == PUPlayerType_AVPlayer) {
            [self.playerUIVC playWithVid:_vid token:_token shouldAutoPlay:options.autoplay];
        }
        else {
            [self.playerUIVC playWithVid:_vid token:_token shouldAutoPlay:options.autoplay needEncrypt:options.encryptEnabled];
        }
    }
}

#pragma mark - notify

- (void)addNotify {
    [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playerPlayStateChanged:)
                                                 name:PMPlayStateChangeNotification object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playerPlayStateChanged:)
                                                 name:PKMoviePlayerPlaybackStateDidChangeNotification object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(playADVideo)
                                                 name:PMPlayerADHeaderOrTailWilPlayNotification object:self.playerUIVC.playerManager];
}

- (void)playADVideo {
    [self.playerUIVC.playerManager startAD];
}

- (void)playerPlayStateChanged:(NSNotification *)noti {
    if([noti.name isEqualToString:PMPlayStateChangeNotification]) {
        BJPlayerManager *playerManager = (BJPlayerManager*)noti.object;
        if(playerManager.playState == PMPlayStatePlaying) {
            //        UIWindow *keyWindow = [UIApplication sharedApplication].keyWindow;
            //        [MBProgressHUD bjp_showMessageThenHide:playerManager.player.contentURL.absoluteString toView:keyWindow onHide:nil];
        }
    }

    if([noti.name isEqualToString:PKMoviePlayerPlaybackStateDidChangeNotification]) {
        PKMoviePlayerController *vc = (PKMoviePlayerController *)noti.object;
        if (vc.playbackState == PKMoviePlaybackStateStopped) {
            NSLog(@"xinyapeng: vc.playbackState = PKMoviePlaybackStateStopped");
        }
        else if (vc.playbackState == PKMoviePlaybackStatePlaying) {
            NSLog(@"xinyapeng: vc.playbackState = PKMoviePlaybackStatePlaying");
        }
        else if (vc.playbackState == PKMoviePlaybackStateInterrupted) {
            NSLog(@"xinyapeng: vc.playbackState = PKMoviePlaybackStateInterrupted");
        }
        else if (vc.playbackState == PKMoviePlaybackStateSeekingForward) {
            NSLog(@"xinyapeng: vc.playbackState = PKMoviePlaybackStateSeekingForward");
        }
        else if (vc.playbackState == PKMoviePlaybackStateSeekingBackward) {
            NSLog(@"xinyapeng: vc.playbackState = PKMoviePlaybackStateSeekingBackward");
        }
    }
}

@end
